#alpha_smoothing
# Generate a noisy sine wave and smooth it!
#
# LAST UPDATED by Andy French May 2018

#Import Python numeric and plotting libraries
import numpy as Npy
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

#

#Display welcome message to console
print(" Noisy wave generator with smoothing")
print(" By Dr Andrew French. April 2018")

#Set smoothing parameter
print("\n Define smoothing parameters alpha")
alpha = input( ' alpha = ' )
if alpha =="":
    alpha = 0.5
    print( " alpha = 0.5")
else:
    alpha = float(alpha)

#Generate three sine waves + some random noise

#Sine wave amplitudes
a1 = 1
a2 = 2
a3 = 3

#Sine wave periods /s
T1 = 5
T2 = 8
T3 = 6

#Sine wave phase /radians
p1 = 0
p2 = Npy.pi/6
p3 = Npy.pi/2

#Generate time /s values
Tmax = 30
N = 1000
t = Npy.linspace(0,Tmax,N)

#Generate sine waves
y = a1*Npy.sin( 2*Npy.pi*t/T1 ) + a2*Npy.sin( 2*Npy.pi*t/T2 ) + a3*Npy.sin( 2*Npy.pi*t/T3 )

#Generate random noise
print( "\n Define noise amplitude" )
noise_amplitude = input( ' Noise amplitude = ' )
if noise_amplitude=="":
    noise_amplitude = 2
    print( " Noise amplitude  = 2 ")
else:
    noise_amplitude = float(noise_amplitude)
        

#Add noise to signal
y_noisy = y + noise_amplitude* ( 2*Npy.random.rand(1,N) - 1 )[0]

#Initialize smoothed signal and alpha smothing value s
ysmooth = [y_noisy[0]]*N
s = y_noisy[0]

#Loop through noisy signal and perform smoothing
for i in range(1,N):
    ysmooth[i] = alpha*s + (1-alpha)*y_noisy[i]
    s = ysmooth[i]

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot y vs t
fsize = 18
plt.plot( t,y_noisy,'+',label = "signal  + noise" , color = [1,0,0] )
plt.plot( t,ysmooth,'-',label = "smoothed signal" , color = [0,1,0] )
plt.xlabel( "t /s", size=fsize*1.2 );
plt.ylabel( "y", size=fsize*1.2 );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( "Alpha smoothed signal alpha = " + str(alpha), size=fsize*1.2 )
plt.legend( fontsize = fsize*1.2,loc = "lower right", numpoints=1 )
plt.grid(True)
plt.xlim( 0, Tmax )

#Export PNG file
fig.savefig('smoothed signal.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press any key to continue ...")

#End of code
