#analogue_to_digital
# Generate a noisy sine wave, and then form a digital signal
# based upon a fixed threshold.
#
# LAST UPDATED by Andy French December 2017

#Import Python numeric and plotting libraries
import numpy as Npy
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

#

#Display welcome message to console
print(" Noisy wave generator with thresholding")
print(" By Dr Andrew French. April 2018")

#Set threshold level
thresh = 1

#Set plot levels of high and low digital values
y_high = 6
y_low = -6

#Generate three sine waves + some random noise

#Sine wave amplitudes
a1 = 1
a2 = 2
a3 = 3

#Sine wave periods /s
T1 = 5
T2 = 8
T3 = 6

#Sine wave phase /radians
p1 = 0
p2 = Npy.pi/6
p3 = Npy.pi/2

#Generate time /s values
Tmax = 30
N = 1000
t = Npy.linspace(0,Tmax,N)

#Generate sine waves
y = a1*Npy.sin( 2*Npy.pi*t/T1 ) + a2*Npy.sin( 2*Npy.pi*t/T2 ) + a3*Npy.sin( 2*Npy.pi*t/T3 )

#Generate random noise
noise_amplitude = 2
yrand = noise_amplitude* ( 2*Npy.random.rand(1,N) - 1 )

#Add noise to signal
y_noisy = y + yrand[0]

#Step through every element of our noisy signal and work out
#the digital value
y_digital = [y_high]*N
for i in range(0,N):
    if y_noisy[i] < thresh:
        y_digital[i] = y_low

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot y vs t
fsize = 18
plt.plot( t,y_noisy,'-',label = "signal  + noise" , color = [1,0,0] )
plt.plot( t,y_digital,'-',label = "digital signal" , color = [0,0,1] )
plt.plot( [0,Tmax], [thresh,thresh], color = [0,1,0] )
plt.xlabel( "t /s", size=fsize*1.2 );
plt.ylabel( "y", size=fsize*1.2 );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( "Signal + noise", size=fsize*1.2 )
plt.legend( fontsize = fsize*1.2,loc = "lower right", numpoints=1 )
plt.grid(True)
plt.xlim( 0, Tmax )

#Export PNG file
fig.savefig('noisy signal + digital.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press any key to continue ...")

#End of code
