#gen_noisy_waves
# Generate the sum of a few sine waves of different amplitudes and
# frequencies + random noise. These are then plotted.
#
# LAST UPDATED by Andy French December 2017

#Import Python numeric and plotting libraries
import numpy as Npy
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

#

#Display welcome message to console
print(" Nosiy wave generator")
print(" By Dr Andrew French. April 2018")

#Generate three sine waves + some random noise

#Sine wave amplitudes
a1 = 1
a2 = 2
a3 = 3

#Sine wave periods /s
T1 = 5
T2 = 8
T3 = 6

#Sine wave phase /radians
p1 = 0
p2 = Npy.pi/6
p3 = Npy.pi/2

#Generate time /s values
Tmax = 30
N = 100
t = Npy.linspace(0,Tmax,N)

#Generate sine waves
y = a1*Npy.sin( 2*Npy.pi*t/T1 ) + a2*Npy.sin( 2*Npy.pi*t/T2 ) + a3*Npy.sin( 2*Npy.pi*t/T3 )

#Generate random noise
noise_amplitude = 1
yrand = noise_amplitude* ( 2*Npy.random.rand(1,N) - 1 )

#Add noise to signal
yrand = yrand[0]

#Create figure and axis
fig = plt.figure(figsize=(10, 8))

#Plot y vs t
fsize = 18
plt.plot( t,y,'-',label = "signal" , color = [1,0,0] )
plt.plot( t,y + yrand,'+',label = "signal  + noise" , color = [1,0,0] )
plt.xlabel( "t /s", size=fsize*1.2 );
plt.ylabel( "y", size=fsize*1.2 );
plt.xticks( fontsize = fsize )
plt.yticks( fontsize = fsize )
plt.title( "Signal + noise", size=fsize*1.2 )
plt.legend( fontsize = fsize*1.2,loc = "lower right", numpoints=1 )
plt.grid(True)
plt.xlim( 0, Tmax )

#Export PNG file
fig.savefig('noisy signal.png', dpi=300)

#Wait for user input to close terminal window
input("\n Press any key to continue ...")

#End of code
